import subprocess

var("x,y")


def l_lag(mu):
	lag = [0,0,0, ((1-2*mu)/2, sqrt(3)/2), ((1-2*mu)/2, -sqrt(3)/2) ]
	fu = (1-mu)*(x-1+mu)^2*((x+mu)^3-1)+mu*(x+mu)^2*((x-1+mu)^3+1)
	lag[0] = (find_root(fu, -mu,1-mu) ,0)

	fu = (1-mu)*(x-1+mu)^2*((x+mu)^3-1)+mu*(x+mu)^2*((x-1+mu)^3-1)
	lag[1] = (find_root(fu, 1-mu, 2) ,0)

	fu = (1-mu)*(x-1+mu)^2*((x+mu)^3+1)+mu*(x+mu)^2*((x-1+mu)^3+1)
	lag[2] = (find_root(fu, -2, -mu) , 0)
	
	return lag

def lag_p_m(mu):
	P = points([(0,0),(1,0)],size=10)
	P += point(l_lag(mu), size=60, color='red')
	P += point([(-mu,0), (1-mu,0)], size=60, color='green')
	return P



def run_and_plot(Lp=[-1.6,1.6,-1.6,1.6, './r3b.eps']):
	subprocess.call('./run_r3b', shell=True)

	print 'FILE CREATED'

	Ldat = []
	with open('./r3b.dat', 'r') as f:
		res_x = float((Lp[1]-Lp[0])/800.0)
		res_y = float((Lp[3]-Lp[2])/800.0)
		lxy = (10, 10)
		line=f.readline()
		mmu = float(line)
		line=f.readline()
		cJ = float(line)
		line=f.readline()
		nsteps = int(line)
		for k in range(nsteps):
			line=f.readline()
			xy = line.split(',')
			xy = (float(xy[0]),float(xy[1]))
			if abs(lxy[0]-xy[0])/res_x + abs(lxy[1]-xy[1])/res_y > 1:
				lxy = xy
				Ldat.append( xy )
	print 'DATA LOADED IN LIST'
	P = lag_p_m(mmu)
	f = x^2 + y^2 +2*((1-mmu)/sqrt((x+mmu)^2+y^2)+mmu/sqrt((x-1+mmu)^2+y^2))
	P += implicit_plot(f-cJ, (x,Lp[0],Lp[1]), (y,Lp[2],Lp[3]), plot_points=400, linestyle='--', color='red')
	P += list_plot(Ldat, plotjoined=True, thickness=1)
	#P += list_plot(Ldat)
	P.set_aspect_ratio(1)
	#P.axes(False)
	P.fontsize(20)
	P.save(Lp[4], xmin=Lp[0], xmax=Lp[1], ymin=Lp[2], ymax=Lp[3])



def data_to_file(Lor, Lnu):
	names = ['MU','X0','Y0','VX0','VY0', 'N_ORB','DT']
	if (len(Lor)!=5) or (len(Lnu)!=2): 
		print 'Error in the number of parameters of the orbit'
		return
	with open('./r3b.h', 'w') as f:
		f.write('// AUTO-GENERATED FILE\n\n')
		f.write('/* Parameters of the orbit */\n')
		for k in range(5):
			f.write('#define '+names[k]+' '+str(Lor[k])+'\n')
		f.write('\n/* Parameters of the numerical method */\n')
		for k in range(2):
			f.write('#define '+names[k+5]+' '+str(Lnu[k])+'\n')

def orbit( Lor, Lnu, Lpl=[-1.6,1.6,-1.6,1.6,'./r3b.eps']):
	data_to_file(Lor, Lnu)
	run_and_plot(Lpl)

	

# HORSESHOE
Lorb = [0.000953875,	-0.97668, 0.0,	0.0,-0.06118]
Lnum = [30.0, 0.01]
#orbit( Lorb, Lnum )

# ALMOST PERIODIC
Lorb = [0.002,	-0.9, 0.0,	0.0,-0.058]
Lnum = [2.0, 0.05]
#orbit( Lorb, Lnum )

# TADPOLE
Lorb = [0.001,	0.507, 0.874025,	0.0,0.0]
Lnum = [15.5, 0.01]
#orbit( Lorb, Lnum )

# SMALLER TADPOLE
Lorb = [0.01,	0.498, 0.874025,	0.0,0.0]
Lnum = [15.5, 0.01]
Lplo = [0,1.0,-0.1,1.1,'./r3b.eps']
#orbit( Lorb, Lnum, Lplo )

# CHAOS
Lorb = [0.015,	0.493, 0.874025,	0.0,0.0]
Lnum = [15.5, 0.01]
#orbit( Lorb, Lnum )

# LOOPS
Lorb = [0.01,	0.5, 0.8,	0.1,-0.25]
Lnum = [15.5, 0.01]
Lplo = [-1.0,1.0,-1.0,1.0,'./r3b.eps']
#orbit( Lorb, Lnum, Lplo )

# SUCESIVE CAPTURES
Lorb = [0.5,	0.0, 0.0,	0.5,0.0]
Lnum = [5.0, 0.001]
Lplo = [-1.0,1.0,-0.5,0.5,'./r3b.eps']
#orbit( Lorb, Lnum, Lplo )

# ASYMMETRIC CAPTURE
Lorb = [0.4,	0.0, 0.0,	0.9,0.2]
Lnum = [2.5, 0.001]
Lplo = [-1.1,1.3,-0.55,0.55,'./r3b.eps']
#orbit( Lorb, Lnum, Lplo )

# CONFINED CAPTURE
Lorb = [0.355,	0.001, -0.02,	0.9,0.2]
Lnum = [3.0, 0.001]
Lplo = [-0.9,1.05,-0.5,0.5,'./r3b.eps']
#orbit( Lorb, Lnum, Lplo )


# CAPTURE ESCAPE
Lorb = [0.29,	0.1, 0.0,	0.85,0.57]
Lnum = [5.0, 0.0001]
Lplo = [-0.9,1.7,-1.2,0.6,'./r3b.eps']
#orbit( Lorb, Lnum, Lplo )

# ALMOST KEPLER
Lorb = [0.000001,	-1.2, 0.0,	0.0,0.1]
Lnum = [4*3.24, 0.001]
Lplo = [-4,4,-4,4,'./r3b.eps']
#orbit( Lorb, Lnum, Lplo )

# ALMOST KEPLER2
Lorb = [0.01,	-1.2, 0.0,	0.0,0.1]
Lnum = [4*3.24, 0.0001]
Lplo = [-4,4,-4,4,'./r3b.eps']
Lplo = [0.9,1.1,-0.2,0.1,'./r3b.eps']
#orbit( Lorb, Lnum, Lplo )

# ALMOST KEPLER3
Lorb = [0.0099,	-1.2, 0.0,	0.0,0.1]
Lnum = [4*3.24, 0.0001]
Lplo = [-4,4,-4,4,'./r3b.eps']
Lplo = [0.7,1.2,-0.1,0.2,'./r3b.eps']
#orbit( Lorb, Lnum, Lplo )



# PURGE
Lorb = [0.25,	40.2, -0.25,	0.1,0.57]
Lnum = [0.2, 0.1]
orbit( Lorb, Lnum )


subprocess.call('./purge_r3b', shell=True)