import subprocess

var("x,y")



def zero_vel_c(a, L, l=1.6):
	lag = l_lag(a)
	f = x^2 + y^2 +2*((1-a)/sqrt((x+a)^2+y^2)+a/sqrt((x-1+a)^2+y^2))
	P = point([(0,0)],size=0)
	for k in L:
		c = f(x=lag[k][0], y=lag[k][1]) 
		P += implicit_plot(f-c, (x,-l,l), (y,-l,l), plot_points=400)
	return P



def cJa(Lor):
	a = Lor[0]
	f = x^2 + y^2 +2*((1-a)/sqrt((x+a)^2+y^2)+a/sqrt((x-1+a)^2+y^2))
	return	(f(x=Lor[1], y=Lor[2])-Lor[3]^2-Lor[4]^2 ).n()


def c1mu(a):
	lag = l_lag(a)
	f = x^2 + y^2 +2*((1-a)/sqrt((x+a)^2+y^2)+a/sqrt((x-1+a)^2+y^2))
	return	f(x=lag[0][0], y=0) 

def c2mu(a):
	lag = l_lag(a)
	f = x^2 + y^2 +2*((1-a)/sqrt((x+a)^2+y^2)+a/sqrt((x-1+a)^2+y^2))
	return	f(x=lag[1][0], y=0) 

def l_lag(mu):
	lag = [0,0,0, ((1-2*mu)/2, sqrt(3)/2), ((1-2*mu)/2, -sqrt(3)/2) ]
	fu = (1-mu)*(x-1+mu)^2*((x+mu)^3-1)+mu*(x+mu)^2*((x-1+mu)^3+1)
	lag[0] = (find_root(fu, -mu,1-mu) ,0)

	fu = (1-mu)*(x-1+mu)^2*((x+mu)^3-1)+mu*(x+mu)^2*((x-1+mu)^3-1)
	lag[1] = (find_root(fu, 1-mu, 2) ,0)

	fu = (1-mu)*(x-1+mu)^2*((x+mu)^3+1)+mu*(x+mu)^2*((x-1+mu)^3+1)
	lag[2] = (find_root(fu, -2, -mu) , 0)
	
	return lag

def lag_p_m(mu):
	P = points([(0,0),(1,0)],size=0)
	P += point(l_lag(mu), size=20, color='red')
	P += point([(-mu,0), (1-mu,0)], size=50, color='green', zorder =200)
	return P

def single_zv(a, c, l=1.6):
	f = x^2 + y^2 +2*((1-a)/sqrt((x+a)^2+y^2)+a/sqrt((x-1+a)^2+y^2))
	return lag_p_m(a)+implicit_plot(f-c, (x,-l,l), (y,-l,l), plot_points=400)

	
	
	
def run_and_plot(Lp=[-1.6,1.6,-1.6,1.6, './r3b.eps']):
	subprocess.call('./run_r3b', shell=True)

	print 'FILE CREATED'

	Ldat = []
	with open('./r3b.dat', 'r') as f:
		res_x = float((Lp[1]-Lp[0])/800.0)
		res_y = float((Lp[3]-Lp[2])/800.0)
		lxy = (10, 10)
		line=f.readline()
		mmu = float(line)
		line=f.readline()
		cJ = float(line)
		line=f.readline()
		nsteps = int(line)
		for k in range(nsteps):
			line=f.readline()
			xy = line.split(',')
			xy = (float(xy[0]),float(xy[1]))
			if abs(lxy[0]-xy[0])/res_x + abs(lxy[1]-xy[1])/res_y > 1:
				lxy = xy
				Ldat.append( xy )
	print 'DATA LOADED IN LIST'
	P = lag_p_m(mmu)
	f = x^2 + y^2 +2*((1-mmu)/sqrt((x+mmu)^2+y^2)+mmu/sqrt((x-1+mmu)^2+y^2))
	P += implicit_plot(f-cJ, (x,Lp[0],Lp[1]), (y,Lp[2],Lp[3]), plot_points=400, linestyle='--', color='red')
	P += list_plot(Ldat, plotjoined=True, thickness=1)
	#P += list_plot(Ldat)
	P.set_aspect_ratio(1)
	#P.axes(False)
	P.fontsize(15)
	P.save(Lp[4], xmin=Lp[0], xmax=Lp[1], ymin=Lp[2], ymax=Lp[3], figsize=5)

	


def data_to_file(Lor, Lnu):
	names = ['MU','X0','Y0','VX0','VY0', 'N_ORB','DT']
	if (len(Lor)!=5) or (len(Lnu)!=2): 
		print 'Error in the number of parameters of the orbit'
		return
	with open('./r3b.h', 'w') as f:
		f.write('// AUTO-GENERATED FILE\n\n')
		f.write('/* Parameters of the orbit */\n')
		for k in range(5):
			f.write('#define '+names[k]+' '+str(Lor[k])+'\n')
		f.write('\n/* Parameters of the numerical method */\n')
		for k in range(2):
			f.write('#define '+names[k+5]+' '+str(Lnu[k])+'\n')

def orbit( Lor, Lnu, Lpl=[-1.6,1.6,-1.6,1.6,'./r3b.eps']):
	pcs(Lor, Lpl)
	data_to_file(Lor, Lnu)
	run_and_plot(Lpl)

	
	
def pcs(Lor,Lp):	
	print '\n'+Lp[-1]
	print 'Lorb =', [k.n(digits=6) for k in Lor]
	print 'c1 =', c1mu(Lor[0])
	print 'cJ =', cJa(Lor)
	print 'c2 =', c2mu(Lor[0])
	print 'c1-cJ =',c1mu(Lor[0])-cJa(Lor)
	print 'c2-cJ =',c2mu(Lor[0])-cJa(Lor)
	print  '============='




def plot1():
	# per5
	x0, y0 = 0.732035, 0.713203
	Lorb = [0.000953875,  x0,y0,	0.112027,  -0.114985]
	Lnum = [5.0, 0.01]
	Lplo = [-0.8,1,-0.9,0.9,'../../images/pp_c.png']
	orbit( Lorb, Lnum, Lplo )

	# per6
	x0, y0 = 1.1378, 0.0
	Lorb = [0.00007803691,  x0,y0,	0.0,  -0.174265]
	Lnum = [4.2, 0.01]
	Lplo = [-1.5,1.5,-1.5,1.5,'../../images/pp_w.png']
	orbit( Lorb, Lnum, Lplo )

	# PERIODIC trian
	Lorb = [0.002,	-0.9, 0.0,	0.0,-0.058]
	Lnum = [2.0, 0.01]
	Lplo = [-1.2,1.2,-1.2,1.2,'../../images/p_t.png']
	orbit( Lorb, Lnum, Lplo )

	# PERIODIC star 7
	Lorb = [0.2,	0.1, 0.08,	1.0, 0.8]
	Lnum = [1.5, 0.0001]
	Lplo = [-0.9,1.1,-0.5,0.5,'../../images/p_7.png']
	orbit( Lorb, Lnum, Lplo )


def plot2():
	# HORSESHOE a
	Lorb = [0.000953875,	-0.97668, 0.0,	0.0,-0.06118]
	Lnum = [30.0, 0.01]
	Lplo = [-1.2,1.2,-1.2,1.2,'../../images/hs_a.png']
	orbit( Lorb, Lnum, Lplo )
		
	# HORSESHOE b
	Lorb = [0.000953875,	-1.02745, 0.0,	0.0,0.04032]
	Lnum = [30.0, 0.01]
	Lplo = [-1.2,1.2,-1.2,1.2,'../../images/hs_b.png']
	orbit( Lorb, Lnum, Lplo )
	
	# BaMi A(2)
	x0 =   -0.93449350
	vy0 =  -0.14032108
	Lorb = [0.0001,	x0, 0.0,	0.0,vy0]
	Lnum = [68.0, 0.001]
	Lplo = [-1.2,1.2,-1.2,1.2,'../../images/hs_a2.png']
	orbit( Lorb, Lnum, Lplo )
	
	# BaMi C(4)
	x0 =   -1.05221706
	vy0 =   0.07736088
	Lorb = [0.0001,	x0, 0.0,	0.0,vy0]
	Lnum = [18.0, 0.001]
	Lplo = [-1.2,1.2,-1.2,1.2,'../../images/hs_c4.png']
	orbit( Lorb, Lnum, Lplo )

	# BaMi B(7)
	x0 =   -1.06764576
	vy0 =   0.11629385
	Lorb = [0.0001,	x0, 0.0,	0.0,vy0]
	Lnum = [36.3, 0.001]
	Lplo = [-1.2,1.2,-1.2,1.2,'../../images/hs_b7.png']
	orbit( Lorb, Lnum, Lplo )






plot1()
plot2()




# PURGE
Lorb = [0.25,	40.2, -0.25,	0.1,0.57]
Lnum = [0.2, 0.1]
orbit( Lorb, Lnum )

subprocess.call('./purge_r3b', shell=True)
